@@ -31,15 +31,15 @@
 
 # TODO Need to fix ASAP conditions in e4 not comma delimited
 
-__version__ = '1.3.3'
-__date__ = 'June, 2022'
+__version__ = '1.3.7'
+__date__ = 'July, 2022'
 __program__ = 'E5'
 
 # region Imports
+from kivy.config import Config
 from kivy import resources
 from kivy.clock import Clock, mainthread
 from kivy.app import App
-from kivy.uix.camera import Camera
 from kivy.uix.switch import Switch
 from kivy.factory import Factory
 from kivy.uix.popup import Popup
@@ -84,7 +84,7 @@ from e5py.lib.colorscheme import ColorScheme
 from e5py.lib.misc import platform_name, restore_window_size_position, locate_file, filename_only
 
 # The database - pure Python
-# from tinydb import TinyDB, Query, where
+from tinydb import where
 from tinydb import __version__ as __tinydb_version__
 
 
@@ -257,6 +257,69 @@ class cfg(blockdata):
     def validate_current_record(self):
         return(True)
 
+    def camera_in_cfg(self):
+        for field in self.fields():
+            f = self.get(field)
+            if f.inputtype == 'CAMERA':
+                return(True)
+        return(False)
+
+    def validate_datafield(self, data_to_insert, data_table):
+        # This just validates one field (e.g. when an existing record is edit)
+        if data_to_insert and data_table and len(data_to_insert) == 1:
+            for field, value in data_to_insert.items():
+                f = self.get(field)
+                if f.required and value.strip() == "":
+                    error_message = f'\nThe field {field} is set to unique or required.  Enter a value to save this record.'
+                    return(error_message)
+                if f.inputtype == 'NUMERIC':
+                    try:
+                        float(value)
+                    except ValueError:
+                        error_message = f'\nThe field {field} requires a valid number.  Correct to save this record.'
+                        return(error_message)
+                if f.unique:
+                    result = data_table.search(where(field) == value)
+                    if result:
+                        error_message = f'\nThe field {field} is set to unique and the value {value} already exists for this field in this data table.'
+                        return(error_message)
+        return(True)
+
+    def validate_datarecord(self, data_to_insert, data_table):
+        # This validates one record (e.g. one a record is about to be inserted)
+        for field in self.fields():
+            f = self.get(field)
+            if f.required:
+                if field in data_to_insert.keys():
+                    if data_to_insert[field].strip() == '':
+                        error_message = f'\nThe field {field} is set to unique or required.  Enter a value to save this record.'
+                        return(error_message)
+                else:
+                    error_message = f'\nThe field {field} is set to unique or required.  Enter a value to save this record.'
+                    return(error_message)
+            if f.inputtype == 'NUMERIC':
+                if field in data_to_insert.keys():
+                    try:
+                        float(data_to_insert[field])
+                    except ValueError:
+                        error_message = f'\nThe field {field} requires a valid number.  Correct to save this record.'
+                        return(error_message)
+            if f.unique:
+                if field in data_to_insert.keys():
+                    result = data_table.search(where(field) == data_to_insert[field])
+                    if result:
+                        error_message = f'\nThe field {field} is set to unique and the value {data_to_insert[field]} already exists for this field in this data table.'
+                        return(error_message)
+                else:
+                    error_message = f'\nThe field {field} is set to unique and a value was not provided for this field.  Unique fields require a value.'
+                    return(error_message)
+
+        # TODO test to see if it is units, prisms or datums
+        # TODO create a unit, prism or datum from the dictionary using *data_to_insert
+        # TODO run the validator in whichever
+        # TODO and return results
+        return(True)
+
     def get(self, field_name):
         if not field_name:
             return('')
@@ -557,26 +620,38 @@ class cfg(blockdata):
     def format_conditions(self, conditions):
         formatted_conditions = []
         for condition in conditions:
-            condition_parsed = condition.split(' ')
-            condition_parsed = self.clean_menu(condition_parsed)
-            formatted_condition = a_condition()
-            if len(condition_parsed) > 1:
-                formatted_condition.match_field = condition_parsed[0].upper()
-                if condition_parsed[1].upper() == 'NOT':
-                    formatted_condition.negate_it = True
-                    if len(condition_parsed) > 2:
-                        formatted_condition.match_list = condition_parsed[2].upper().split(',')
+            condition_parsed = []
+            if ' ' in condition:
+                condition_parsed.append(condition[:condition.find(' ')])
+                condition = condition[condition.find(' '):].strip()
+                if condition[:3].upper() == 'NOT':
+                    condition_parsed.append('NOT')
+                    condition = condition[3:]
+                if condition.upper().endswith(' OR'):
+                    condition = condition[:-3]
+                    condition_parsed.append(condition.strip())
+                    condition_parsed.append('OR')
                 else:
-                    formatted_condition.match_list = condition_parsed[1].upper().split(',')
-                if formatted_condition.negate_it and len(condition_parsed) == 4:
-                    if condition_parsed[3].upper() == 'OR':
-                        formatted_condition.or_it = True
-                elif not formatted_condition.negate_it and len(condition_parsed) == 3:
-                    if condition_parsed[2].upper() == 'OR':
-                        formatted_condition.or_it = True
-                if formatted_condition.match_list:
-                    formatted_condition.match_list = self.clean_menu(formatted_condition.match_list)
-            formatted_conditions.append(formatted_condition)
+                    condition_parsed.append(condition.strip())
+                condition_parsed = self.clean_menu(condition_parsed)
+                formatted_condition = a_condition()
+                if len(condition_parsed) > 1:
+                    formatted_condition.match_field = condition_parsed[0].upper()
+                    if condition_parsed[1].upper() == 'NOT':
+                        formatted_condition.negate_it = True
+                        if len(condition_parsed) > 2:
+                            formatted_condition.match_list = condition_parsed[2].upper().split(',')
+                    else:
+                        formatted_condition.match_list = condition_parsed[1].upper().split(',')
+                    if formatted_condition.negate_it and len(condition_parsed) == 4:
+                        if condition_parsed[3].upper() == 'OR':
+                            formatted_condition.or_it = True
+                    elif not formatted_condition.negate_it and len(condition_parsed) == 3:
+                        if condition_parsed[2].upper() == 'OR':
+                            formatted_condition.or_it = True
+                    if formatted_condition.match_list:
+                        formatted_condition.match_list = self.clean_menu(formatted_condition.match_list)
+                formatted_conditions.append(formatted_condition)
         return(formatted_conditions)
 
     def gps_location(self):
@@ -794,11 +869,12 @@ class MainScreen(e5_MainScreen):
         super(MainScreen, self).__init__(**kwargs)
 
         self.user_data_dir = user_data_dir
+        self.setup_logger()
+
         self.colors = ColorScheme()
         self.ini = ini()
         self.cfg = cfg()
         self.data = db()
-
         self.setup_program()
 
         if platform_name() == 'Android':
@@ -831,7 +907,7 @@ class MainScreen(e5_MainScreen):
         self.build_mainscreen()
         self.add_screens()
         restore_window_size_position(__program__, self.ini)
-        self.setup_logger()
+        self.if_camera_setup_camera()
 
     def setup_logger(self):
         logger = logging.getLogger(__name__)
@@ -935,6 +1011,16 @@ class MainScreen(e5_MainScreen):
             self.cfg_menu()
         self.update_title()
 
+    def if_camera_setup_camera(self):
+        if self.cfg.camera_in_cfg() and 1 == 0:
+            try:
+                # self.camera = Camera(play = True, size_hint_y = .8, resolution = (-1, -1))
+                pass
+            except Exception as e:
+                print("Oops!", e.__class__, "occurred.")
+        else:
+            self.camera = None
+
     def cfg_menu(self):
 
         self.cfg_files = self.get_files(self.get_path(), 'cfg')
@@ -977,6 +1063,7 @@ class MainScreen(e5_MainScreen):
         self.ini.update(self.colors, self.cfg)
         self.build_mainscreen()
         self.reset_screens()
+        self.if_camera_setup_camera()
 
     def set_new_data_to_true(self, table_name = None):
         if table_name is None:
@@ -1143,13 +1230,16 @@ class MainScreen(e5_MainScreen):
         if menu_exists or info_exists or camera_exists or gps_exists:
 
             if camera_exists:
+                if self.camera is not None:
+                    if self.camera.parent is not None:
+                        self.camera.parent.remove_widget(self.camera)
                 bx = BoxLayout(orientation = 'vertical')
-                bx.add_widget(Camera(play = True, size_hint_y = .8,
-                                         resolution = (320, 160)))
+                # bx.add_widget(self.camera)
+                # self.camera.play = True
                 bx.add_widget(e5_button(text = "Snap",
-                                        id = "snap", selected = True,
+                                        selected = True,
                                         colors = self.colors,
-                                        callback = self.take_photo))
+                                        call_back = self.take_photo))
                 content_area.add_widget(bx)
 
             if gps_exists:
@@ -1228,16 +1318,16 @@ class MainScreen(e5_MainScreen):
         point += "Accuracy = %s" % round(random() * 5 + 3, 3)
         return(point)
 
-    def take_photo(self):
-        camera = self.get_widget_by_id('camera')
-        if camera.play:
+    def take_photo(self, instance):
+        return
+        if self.camera.play:
             try:
-                camera.export_to_png("IMG_%s.png" % self.time_stamp())
-            except:
-                pass
-                # print('camera file save error')
+                self.camera.export_to_png(path.join(self.cfg.path, "IMG_%s.png" % self.datetime_stamp()))
+            except Exception as e:
+                print("Oops!", e.__class__, "occurred.")
+                print('camera file save error')
                 # TODO Replace this with a popup message
-        camera.play = not camera.play
+        self.camera.play = not self.camera.play
 
     def on_pre_enter(self):
         Window.bind(on_key_down = self._on_keyboard_down)
@@ -1249,6 +1339,8 @@ class MainScreen(e5_MainScreen):
             start_path = self.cfg.path
         else:
             start_path = self.ini.get_value(__program__, 'APP_PATH')
+        if not path.exists(start_path):
+            start_path = path.abspath(path.dirname(__file__))
         content = e5_LoadDialog(load = self.load,
                                 cancel = self.dismiss_popup,
                                 start_path = start_path,
@@ -1471,10 +1563,13 @@ class E5App(App):
     def __init__(self, **kwargs):
         super(E5App, self).__init__(**kwargs)
 
-        self.app_path = self.user_data_dir
+        if platform_name() != 'Android':
+            self.app_path = path.abspath(path.dirname(__file__))
+        else:
+            self.app_path = self.user_data_dir
 
     def build(self):
-        sm.add_widget(MainScreen(user_data_dir = self.user_data_dir, name = 'MainScreen'))
+        sm.add_widget(MainScreen(user_data_dir = self.app_path, name = 'MainScreen'))
         sm.current = 'MainScreen'
         self.title = __program__ + " " + __version__
         return(sm)
@@ -1496,4 +1591,5 @@ def resourcePath():
 if __name__ == '__main__':
     # This line goes with the function above
     resources.resource_add_path(resourcePath())  # add this line
+    Config.set('input', 'mouse', 'mouse,multitouch_on_demand')      # Removes red dot
     E5App().run()
